@@ -1,20 +1,21 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2007 Jeffrey Stedfast
+ *  Copyright (C) 2000-2008 Jeffrey Stedfast
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -32,23 +33,35 @@
 #include "gmime-utils.h"
 #include "gmime-stream-mem.h"
 #include "gmime-table-private.h"
+#include "gmime-parse-utils.h"
 
 
+/**
+ * SECTION: gmime-message
+ * @title: GMimeMessage
+ * @short_description: Messages
+ * @see_also:
+ *
+ * A #GMimeMessage represents an rfc822 message.
+ **/
+
 static void g_mime_message_class_init (GMimeMessageClass *klass);
 static void g_mime_message_init (GMimeMessage *message, GMimeMessageClass *klass);
 static void g_mime_message_finalize (GObject *object);
 
 /* GMimeObject class methods */
 static void message_init (GMimeObject *object);
-static void message_add_header (GMimeObject *object, const char *header, const char *value);
+static void message_prepend_header (GMimeObject *object, const char *header, const char *value);
+static void message_append_header (GMimeObject *object, const char *header, const char *value);
 static void message_set_header (GMimeObject *object, const char *header, const char *value);
 static const char *message_get_header (GMimeObject *object, const char *header);
-static void message_remove_header (GMimeObject *object, const char *header);
+static gboolean message_remove_header (GMimeObject *object, const char *header);
 static char *message_get_headers (GMimeObject *object);
 static ssize_t message_write_to_stream (GMimeObject *object, GMimeStream *stream);
 
-static void message_add_recipients_from_string (GMimeMessage *message, char *type, const char *string);
+static void message_add_recipients_from_string (GMimeMessage *message, char *type, const char *str);
 
+static ssize_t write_structured (GMimeStream *stream, const char *name, const char *value);
 static ssize_t write_addrspec (GMimeStream *stream, const char *name, const char *value);
 static ssize_t write_received (GMimeStream *stream, const char *name, const char *value);
 static ssize_t write_subject (GMimeStream *stream, const char *name, const char *value);
@@ -68,7 +81,6 @@ static char *rfc822_headers[] = {
 	"Sender",
 	"To",
 	"Cc",
-	NULL
 };
 
 
@@ -108,10 +120,11 @@ g_mime_message_class_init (GMimeMessageClass *klass)
 	gobject_class->finalize = g_mime_message_finalize;
 	
 	object_class->init = message_init;
-	object_class->add_header = message_add_header;
+	object_class->prepend_header = message_prepend_header;
+	object_class->append_header = message_append_header;
+	object_class->remove_header = message_remove_header;
 	object_class->set_header = message_set_header;
 	object_class->get_header = message_get_header;
-	object_class->remove_header = message_remove_header;
 	object_class->get_headers = message_get_headers;
 	object_class->write_to_stream = message_write_to_stream;
 }
@@ -119,6 +132,8 @@ g_mime_message_class_init (GMimeMessageClass *klass)
 static void
 g_mime_message_init (GMimeMessage *message, GMimeMessageClass *klass)
 {
+	GMimeHeaderList *headers = ((GMimeObject *) message)->headers;
+	
 	message->from = NULL;
 	message->reply_to = NULL;
 	message->recipients = g_hash_table_new (g_str_hash, g_str_equal);
@@ -128,22 +143,22 @@ g_mime_message_init (GMimeMessage *message, GMimeMessageClass *klass)
 	message->message_id = NULL;
 	message->mime_part = NULL;
 	
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Sender", write_addrspec);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "From", write_addrspec);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "To", write_addrspec);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Cc", write_addrspec);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Bcc", write_addrspec);
+	g_mime_header_list_register_writer (headers, "Sender", write_addrspec);
+	g_mime_header_list_register_writer (headers, "From", write_addrspec);
+	g_mime_header_list_register_writer (headers, "To", write_addrspec);
+	g_mime_header_list_register_writer (headers, "Cc", write_addrspec);
+	g_mime_header_list_register_writer (headers, "Bcc", write_addrspec);
 	
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Resent-Sender", write_addrspec);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Resent-From", write_addrspec);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Resent-To", write_addrspec);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Resent-Cc", write_addrspec);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Resent-Bcc", write_addrspec);
+	g_mime_header_list_register_writer (headers, "Resent-Sender", write_addrspec);
+	g_mime_header_list_register_writer (headers, "Resent-From", write_addrspec);
+	g_mime_header_list_register_writer (headers, "Resent-To", write_addrspec);
+	g_mime_header_list_register_writer (headers, "Resent-Cc", write_addrspec);
+	g_mime_header_list_register_writer (headers, "Resent-Bcc", write_addrspec);
 	
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Subject", write_subject);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Received", write_received);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "Message-Id", write_msgid);
-	g_mime_header_register_writer (((GMimeObject *) message)->headers, "References", write_addrspec);
+	g_mime_header_list_register_writer (headers, "Subject", write_subject);
+	g_mime_header_list_register_writer (headers, "Received", write_received);
+	g_mime_header_list_register_writer (headers, "Message-Id", write_msgid);
+	g_mime_header_list_register_writer (headers, "References", write_structured);
 }
 
 static gboolean
@@ -196,8 +211,6 @@ struct _received_token {
 	token_skip_t skip;
 };
 
-extern void decode_lwsp   (const char **in);
-
 static void skip_atom     (const char **in);
 static void skip_domain   (const char **in);
 static void skip_addr     (const char **in);
@@ -414,10 +427,11 @@ static ssize_t
 write_received (GMimeStream *stream, const char *name, const char *value)
 {
 	struct _received_part *parts, *part, *tail;
-	const char *lwsp, *inptr;
+	const char *inptr, *lwsp = NULL;
 	ssize_t nwritten;
 	GString *str;
-	int len, i;
+	size_t len;
+	guint i;
 	
 	while (is_lwsp (*value))
 		value++;
@@ -437,6 +451,7 @@ write_received (GMimeStream *stream, const char *name, const char *value)
 		for (i = 0; i < G_N_ELEMENTS (received_tokens); i++) {
 			if (!strncmp (inptr, received_tokens[i].token, received_tokens[i].len)) {
 				if (inptr > part->start) {
+					g_assert (lwsp != NULL);
 					part->len = lwsp - part->start;
 					
 					part = g_alloca (sizeof (struct _received_part));
@@ -546,7 +561,7 @@ write_msgid (GMimeStream *stream, const char *name, const char *value)
 }
 
 static ssize_t
-write_addrspec (GMimeStream *stream, const char *name, const char *value)
+write_structured (GMimeStream *stream, const char *name, const char *value)
 {
 	char *unfolded, *folded;
 	ssize_t n;
@@ -561,6 +576,29 @@ write_addrspec (GMimeStream *stream, const char *name, const char *value)
 	return n;
 }
 
+static ssize_t
+write_addrspec (GMimeStream *stream, const char *name, const char *value)
+{
+	InternetAddressList *addrlist;
+	GString *str;
+	ssize_t n;
+	
+	str = g_string_new (name);
+	g_string_append (str, ": ");
+	
+	if (value && (addrlist = internet_address_parse_string (value))) {
+		internet_address_list_writer (addrlist, str);
+		internet_address_list_destroy (addrlist);
+	}
+	
+	g_string_append_c (str, '\n');
+	
+	n = g_mime_stream_write (stream, str->str, str->len);
+	g_string_free (str, TRUE);
+	
+	return n;
+}
+
 
 enum {
 	HEADER_FROM,
@@ -574,7 +612,7 @@ enum {
 	HEADER_UNKNOWN
 };
 
-static char *headers[] = {
+static const char *message_headers[] = {
 	"From",
 	"Reply-To",
 	"To",
@@ -583,7 +621,6 @@ static char *headers[] = {
 	"Subject",
 	"Date",
 	"Message-Id",
-	NULL
 };
 
 static gboolean
@@ -594,8 +631,8 @@ process_header (GMimeObject *object, const char *header, const char *value)
 	int offset, i;
 	time_t date;
 	
-	for (i = 0; headers[i]; i++) {
-		if (!g_ascii_strcasecmp (headers[i], header))
+	for (i = 0; i < G_N_ELEMENTS (message_headers); i++) {
+		if (!g_ascii_strcasecmp (message_headers[i], header))
 			break;
 	}
 	
@@ -638,70 +675,96 @@ process_header (GMimeObject *object, const char *header, const char *value)
 		break;
 	default:
 		return FALSE;
-		break;
 	}
 	
 	return TRUE;
 }
 
 static void
-message_add_header (GMimeObject *object, const char *header, const char *value)
+message_prepend_header (GMimeObject *object, const char *header, const char *value)
 {
+	GMimeMessage *message = (GMimeMessage *) object;
+	
 	if (!g_ascii_strcasecmp ("MIME-Version", header))
 		return;
 	
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
+	if (!g_ascii_strncasecmp ("Content-", header, 8))
+		return;
 	
-	if (g_ascii_strncasecmp ("Content-", header, 8)) {
-		if (process_header (object, header, value))
-			g_mime_header_add (object->headers, header, value);
-		else
-			GMIME_OBJECT_CLASS (parent_class)->add_header (object, header, value);
-	}
+	if (!process_header (object, header, value))
+		GMIME_OBJECT_CLASS (parent_class)->prepend_header (object, header, value);
+	else
+		g_mime_header_list_prepend (object->headers, header, value);
 	
-	if (((GMimeMessage *) object)->mime_part)
-		g_mime_header_set_raw (((GMimeMessage *) object)->mime_part->headers, NULL);
+	if (message->mime_part)
+		g_mime_header_list_set_raw (message->mime_part->headers, NULL);
+}
+
+static void
+message_append_header (GMimeObject *object, const char *header, const char *value)
+{
+	GMimeMessage *message = (GMimeMessage *) object;
+	
+	if (!g_ascii_strcasecmp ("MIME-Version", header))
+		return;
+	
+	/* Make sure that the header is not a Content-* header, else it
+           doesn't belong on a message */
+	if (!g_ascii_strncasecmp ("Content-", header, 8))
+		return;
+	
+	if (!process_header (object, header, value))
+		GMIME_OBJECT_CLASS (parent_class)->append_header (object, header, value);
+	else
+		g_mime_header_list_append (object->headers, header, value);
+	
+	if (message->mime_part)
+		g_mime_header_list_set_raw (message->mime_part->headers, NULL);
 }
 
 static void
 message_set_header (GMimeObject *object, const char *header, const char *value)
 {
+	GMimeMessage *message = (GMimeMessage *) object;
+	
 	if (!g_ascii_strcasecmp ("MIME-Version", header))
 		return;
 	
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
+	if (!g_ascii_strncasecmp ("Content-", header, 8))
+		return;
 	
-	if (g_ascii_strncasecmp ("Content-", header, 8)) {
-		if (process_header (object, header, value))
-			g_mime_header_set (object->headers, header, value);
-		else
-			GMIME_OBJECT_CLASS (parent_class)->set_header (object, header, value);
-	}
+	if (!process_header (object, header, value))
+		GMIME_OBJECT_CLASS (parent_class)->set_header (object, header, value);
+	else
+		g_mime_header_list_set (object->headers, header, value);
 	
-	if (((GMimeMessage *) object)->mime_part)
-		g_mime_header_set_raw (((GMimeMessage *) object)->mime_part->headers, NULL);
+	if (message->mime_part)
+		g_mime_header_list_set_raw (message->mime_part->headers, NULL);
 }
 
 static const char *
 message_get_header (GMimeObject *object, const char *header)
 {
-	/* Make sure that the header is not a Content-* header, else it
-           doesn't belong on a message */
+	GMimeMessage *message = (GMimeMessage *) object;
 	
 	if (!g_ascii_strcasecmp ("MIME-Version", header))
 		return "1.0";
 	
+	/* Make sure that the header is not a Content-* header, else it
+           doesn't belong on a message */
 	if (g_ascii_strncasecmp ("Content-", header, 8) != 0)
 		return GMIME_OBJECT_CLASS (parent_class)->get_header (object, header);
-	else if (((GMimeMessage *) object)->mime_part)
-		return g_mime_object_get_header (((GMimeMessage *) object)->mime_part, header);
+	else if (message->mime_part)
+		return g_mime_object_get_header (message->mime_part, header);
 	else
 		return NULL;
 }
 
-static void
+static gboolean
 message_remove_header (GMimeObject *object, const char *header)
 {
 	GMimeMessage *message = (GMimeMessage *) object;
@@ -709,16 +772,19 @@ message_remove_header (GMimeObject *object, const char *header)
 	int i;
 	
 	if (!g_ascii_strcasecmp ("MIME-Version", header))
-		return;
+		return FALSE;
 	
 	/* Make sure that the header is not a Content-* header, else it
            doesn't belong on a message */
+	if (!g_ascii_strncasecmp ("Content-", header, 8)) {
+		if (message->mime_part)
+			return g_mime_object_remove_header (message->mime_part, header);
+		
+		return FALSE;
+	}
 	
-	if (!g_ascii_strncasecmp ("Content-", header, 8))
-		return;
-	
-	for (i = 0; headers[i]; i++) {
-		if (!g_ascii_strcasecmp (headers[i], header))
+	for (i = 0; i < G_N_ELEMENTS (message_headers); i++) {
+		if (!g_ascii_strcasecmp (message_headers[i], header))
 			break;
 	}
 	
@@ -762,10 +828,10 @@ message_remove_header (GMimeObject *object, const char *header)
 		break;
 	}
 	
-	GMIME_OBJECT_CLASS (parent_class)->remove_header (object, header);
+	if (message->mime_part)
+		g_mime_header_list_set_raw (message->mime_part->headers, NULL);
 	
-	if (((GMimeMessage *) object)->mime_part)
-		g_mime_header_set_raw (((GMimeMessage *) object)->mime_part->headers, NULL);
+	return GMIME_OBJECT_CLASS (parent_class)->remove_header (object, header);
 }
 
 
@@ -781,20 +847,20 @@ message_get_headers (GMimeObject *object)
 	stream = g_mime_stream_mem_new ();
 	g_mime_stream_mem_set_byte_array (GMIME_STREAM_MEM (stream), ba);
 	
-	if (message->mime_part && g_mime_header_has_raw (message->mime_part->headers)) {
+	if (message->mime_part && g_mime_header_list_has_raw (message->mime_part->headers)) {
 		/* if the mime part has raw headers, then it contains the message headers as well */
-		g_mime_header_write_to_stream (message->mime_part->headers, stream);
+		g_mime_header_list_write_to_stream (message->mime_part->headers, stream);
 	} else {
-		g_mime_header_write_to_stream (object->headers, stream);
+		g_mime_header_list_write_to_stream (object->headers, stream);
 		if (message->mime_part) {
 			g_mime_stream_write_string (stream, "MIME-Version: 1.0\n");
-			g_mime_header_write_to_stream (message->mime_part->headers, stream);
+			g_mime_header_list_write_to_stream (message->mime_part->headers, stream);
 		}
 	}
 	
 	g_object_unref (stream);
-	g_byte_array_append (ba, "", 1);
-	str = ba->data;
+	g_byte_array_append (ba, (unsigned char *) "", 1);
+	str = (char *) ba->data;
 	g_byte_array_free (ba, FALSE);
 	
 	return str;
@@ -806,24 +872,29 @@ message_write_to_stream (GMimeObject *object, GMimeStream *stream)
 	GMimeMessage *message = (GMimeMessage *) object;
 	ssize_t nwritten, total = 0;
 	
-	if (!(message->mime_part && g_mime_header_has_raw (message->mime_part->headers))) {
-		/* write the content headers */
-		if ((nwritten = g_mime_header_write_to_stream (object->headers, stream)) == -1)
-			return -1;
-		
-		total += nwritten;
-		
-		if (message->mime_part) {
+	if (message->mime_part) {
+		if (!g_mime_header_list_has_raw (message->mime_part->headers)) {
+			/* if the mime part has raw headers, then it contains the message
+			 * headers as well otherwise it doesn't, so write them now */
+			if ((nwritten = g_mime_header_list_write_to_stream (object->headers, stream)) == -1)
+				return -1;
+			
+			total += nwritten;
+			
 			if ((nwritten = g_mime_stream_write_string (stream, "MIME-Version: 1.0\n")) == -1)
 				return -1;
 			
 			total += nwritten;
 		}
-	}
-	
-	if (message->mime_part) {
-		nwritten = g_mime_object_write_to_stream (message->mime_part, stream);
+		
+		if ((nwritten = g_mime_object_write_to_stream (message->mime_part, stream)) == -1)
+			return -1;
 	} else {
+		if ((nwritten = g_mime_header_list_write_to_stream (object->headers, stream)) == -1)
+			return -1;
+		
+		total += nwritten;
+		
 		if ((nwritten = g_mime_stream_write (stream, "\n", 1)) == -1)
 			return -1;
 	}
@@ -843,20 +914,22 @@ message_write_to_stream (GMimeObject *object, GMimeStream *stream)
  * strictly a cosmetic thing, so if you are unsure, it is safe to say
  * no (%FALSE).
  *
- * Returns an empty MIME Message object.
+ * Returns: an empty MIME Message object.
  **/
 GMimeMessage *
 g_mime_message_new (gboolean pretty_headers)
 {
+	GMimeHeaderList *headers;
 	GMimeMessage *message;
 	int i;
 	
-	message = g_object_new (GMIME_TYPE_MESSAGE, NULL, NULL);
+	message = g_object_new (GMIME_TYPE_MESSAGE, NULL);
 	
 	if (pretty_headers) {
 		/* Populate with the "standard" rfc822 headers so we can have a standard order */
-		for (i = 0; rfc822_headers[i]; i++) 
-			g_mime_header_set (GMIME_OBJECT (message)->headers, rfc822_headers[i], NULL);
+		headers = ((GMimeObject *) message)->headers;
+		for (i = 0; i < G_N_ELEMENTS (rfc822_headers); i++) 
+			g_mime_header_list_set (headers, rfc822_headers[i], NULL);
 	}
 	
 	return message;
@@ -887,7 +960,7 @@ g_mime_message_set_sender (GMimeMessage *message, const char *sender)
 	encoded = internet_address_list_to_string (addrlist, TRUE);
 	internet_address_list_destroy (addrlist);
 	
-	g_mime_header_set (GMIME_OBJECT (message)->headers, "From", encoded);
+	g_mime_header_list_set (GMIME_OBJECT (message)->headers, "From", encoded);
 	g_free (encoded);
 }
 
@@ -898,7 +971,7 @@ g_mime_message_set_sender (GMimeMessage *message, const char *sender)
  *
  * Gets the email address of the sender from @message.
  *
- * Returns the sender's name and address of the MIME Message.
+ * Returns: the sender's name and address of the MIME Message.
  **/
 const char *
 g_mime_message_get_sender (GMimeMessage *message)
@@ -925,7 +998,7 @@ g_mime_message_set_reply_to (GMimeMessage *message, const char *reply_to)
 	g_free (message->reply_to);
 	message->reply_to = g_strstrip (g_strdup (reply_to));
 	
-	g_mime_header_set (GMIME_OBJECT (message)->headers, "Reply-To", message->reply_to);
+	g_mime_header_list_set (GMIME_OBJECT (message)->headers, "Reply-To", message->reply_to);
 }
 
 
@@ -935,7 +1008,7 @@ g_mime_message_set_reply_to (GMimeMessage *message, const char *reply_to)
  *
  * Gets the Reply-To address from @message.
  *
- * Returns the sender's Reply-To address from the MIME Message.
+ * Returns: the sender's Reply-To address from the MIME Message.
  **/
 const char *
 g_mime_message_get_reply_to (GMimeMessage *message)
@@ -950,17 +1023,15 @@ static void
 sync_recipient_header (GMimeMessage *message, const char *type)
 {
 	const InternetAddressList *recipients;
+	char *string;
 	
 	/* sync the specified recipient header */
-	recipients = g_mime_message_get_recipients (message, type);
-	if (recipients) {
-		char *string;
-		
+	if ((recipients = g_mime_message_get_recipients (message, type))) {
 		string = internet_address_list_to_string (recipients, TRUE);
-		g_mime_header_set (GMIME_OBJECT (message)->headers, type, string);
+		g_mime_header_list_set (GMIME_OBJECT (message)->headers, type, string);
 		g_free (string);
 	} else
-		g_mime_header_set (GMIME_OBJECT (message)->headers, type, NULL);
+		g_mime_header_list_set (GMIME_OBJECT (message)->headers, type, NULL);
 }
 
 
@@ -1000,15 +1071,14 @@ g_mime_message_add_recipient (GMimeMessage *message, char *type, const char *nam
 
 
 static void
-message_add_recipients_from_string (GMimeMessage *message, char *type, const char *string)
+message_add_recipients_from_string (GMimeMessage *message, char *type, const char *str)
 {
 	InternetAddressList *recipients, *addrlist;
 	
 	recipients = g_hash_table_lookup (message->recipients, type);
 	g_hash_table_remove (message->recipients, type);
 	
-	addrlist = internet_address_parse_string (string);
-	if (addrlist) {
+	if ((addrlist = internet_address_parse_string (str))) {
 		recipients = internet_address_list_concat (recipients, addrlist);
 		internet_address_list_destroy (addrlist);
 	}
@@ -1021,7 +1091,7 @@ message_add_recipients_from_string (GMimeMessage *message, char *type, const cha
  * g_mime_message_add_recipients_from_string:
  * @message: MIME Message
  * @type: Recipient type
- * @string: A string of recipient names and addresses.
+ * @str: A string of recipient names and addresses.
  *
  * Add a list of recipients of a chosen type to the MIME
  * Message. Available recipient types include:
@@ -1030,13 +1100,13 @@ message_add_recipients_from_string (GMimeMessage *message, char *type, const cha
  * specified in rfc822.
  **/
 void
-g_mime_message_add_recipients_from_string (GMimeMessage *message, char *type, const char *string)
+g_mime_message_add_recipients_from_string (GMimeMessage *message, char *type, const char *str)
 {
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
 	g_return_if_fail (type != NULL);
-	g_return_if_fail (string != NULL);
+	g_return_if_fail (str != NULL);
 	
-	message_add_recipients_from_string (message, type, string);
+	message_add_recipients_from_string (message, type, str);
 	
 	sync_recipient_header (message, type);
 }
@@ -1051,7 +1121,7 @@ g_mime_message_add_recipients_from_string (GMimeMessage *message, char *type, co
  * recipient types include: #GMIME_RECIPIENT_TYPE_TO,
  * #GMIME_RECIPIENT_TYPE_CC and #GMIME_RECIPIENT_TYPE_BCC.
  *
- * Returns a list of recipients of a chosen type from the MIME
+ * Returns: a list of recipients of a chosen type from the MIME
  * Message.
  **/
 const InternetAddressList *
@@ -1064,6 +1134,50 @@ g_mime_message_get_recipients (GMimeMessage *message, const char *type)
 }
 
 
+static const char *recipient_types[] = {
+	GMIME_RECIPIENT_TYPE_TO,
+	GMIME_RECIPIENT_TYPE_CC,
+	GMIME_RECIPIENT_TYPE_BCC
+};
+
+/**
+ * g_mime_message_get_all_recipients:
+ * @message: MIME Message
+ *
+ * Gets the complete list of recipients for @message.
+ *
+ * Returns: a newly allocated #InternetAddressList containing all
+ * recipients of the message.
+ **/
+InternetAddressList *
+g_mime_message_get_all_recipients (GMimeMessage *message)
+{
+	InternetAddressList *list, *tail, *node, *recipients;
+	guint i;
+	
+	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
+	
+	list = NULL;
+	tail = (InternetAddressList *) &list;
+	
+	for (i = 0; i < G_N_ELEMENTS (recipient_types); i++) {
+		recipients = g_hash_table_lookup (message->recipients, recipient_types[i]);
+		while (recipients != NULL) {
+			internet_address_ref (recipients->address);
+			node = g_new (InternetAddressList, 1);
+			node->address = recipients->address;
+			node->next = NULL;
+			tail->next = node;
+			tail = node;
+			
+			recipients = recipients->next;
+		}
+	}
+	
+	return list;
+}
+
+
 /**
  * g_mime_message_set_subject:
  * @message: MIME Message
@@ -1083,7 +1197,7 @@ g_mime_message_set_subject (GMimeMessage *message, const char *subject)
 	message->subject = g_strstrip (g_strdup (subject));
 	
 	encoded = g_mime_utils_header_encode_text (message->subject);
-	g_mime_header_set (GMIME_OBJECT (message)->headers, "Subject", encoded);
+	g_mime_object_set_header (GMIME_OBJECT (message), "Subject", encoded);
 	g_free (encoded);
 }
 
@@ -1094,7 +1208,7 @@ g_mime_message_set_subject (GMimeMessage *message, const char *subject)
  *
  * Gets the message's subject.
  *
- * Returns the unencoded UTF-8 Subject field on a MIME Message.
+ * Returns: the unencoded UTF-8 Subject field on a MIME Message.
  **/
 const char *
 g_mime_message_get_subject (GMimeMessage *message)
@@ -1124,7 +1238,7 @@ g_mime_message_set_date (GMimeMessage *message, time_t date, int gmt_offset)
 	message->gmt_offset = gmt_offset;
 	
 	date_str = g_mime_message_get_date_string (message);
-	g_mime_header_set (GMIME_OBJECT (message)->headers, "Date", date_str);
+	g_mime_object_set_header (GMIME_OBJECT (message), "Date", date_str);
 	g_free (date_str);
 }
 
@@ -1157,7 +1271,7 @@ g_mime_message_get_date (GMimeMessage *message, time_t *date, int *gmt_offset)
  *
  * Gets the message's sent date in string format.
  * 
- * Returns the sent-date of the MIME Message in string format.
+ * Returns: the sent-date of the MIME Message in string format.
  **/
 char *
 g_mime_message_get_date_string (GMimeMessage *message)
@@ -1192,7 +1306,7 @@ g_mime_message_set_message_id (GMimeMessage *message, const char *message_id)
 	message->message_id = g_strstrip (g_strdup (message_id));
 	
 	msgid = g_strdup_printf ("<%s>", message_id);
-	g_mime_header_set (GMIME_OBJECT (message)->headers, "Message-Id", msgid);
+	g_mime_object_set_header (GMIME_OBJECT (message), "Message-Id", msgid);
 	g_free (msgid);
 }
 
@@ -1203,7 +1317,7 @@ g_mime_message_set_message_id (GMimeMessage *message, const char *message_id)
  *
  * Gets the Message-Id header of @message.
  *
- * Returns the Message-Id of a message.
+ * Returns: the Message-Id of a message.
  **/
 const char *
 g_mime_message_get_message_id (GMimeMessage *message)
@@ -1214,73 +1328,13 @@ g_mime_message_get_message_id (GMimeMessage *message)
 }
 
 
-/**
- * g_mime_message_add_header: Add an arbitrary message header
- * @message: MIME Message
- * @header: rfc822 header field
- * @value: the contents of the header field
- *
- * Add an arbitrary message header to the MIME Message such as X-Mailer,
- * X-Priority, or In-Reply-To.
- **/
-void
-g_mime_message_add_header (GMimeMessage *message, const char *header, const char *value)
-{
-	g_return_if_fail (GMIME_IS_MESSAGE (message));
-	g_return_if_fail (header != NULL);
-	g_return_if_fail (value != NULL);
-	
-	g_mime_object_add_header (GMIME_OBJECT (message), header, value);
-}
-
-
-/**
- * g_mime_message_set_header: Add an arbitrary message header
- * @message: MIME Message
- * @header: rfc822 header field
- * @value: the contents of the header field
- *
- * Set an arbitrary message header to the MIME Message such as X-Mailer,
- * X-Priority, or In-Reply-To.
- **/
-void
-g_mime_message_set_header (GMimeMessage *message, const char *header, const char *value)
-{
-	g_return_if_fail (GMIME_IS_MESSAGE (message));
-	g_return_if_fail (header != NULL);
-	g_return_if_fail (value != NULL);
-	
-	g_mime_object_set_header (GMIME_OBJECT (message), header, value);
-}
-
-
-/**
- * g_mime_message_get_header:
- * @message: MIME Message
- * @header: rfc822 header field
- *
- * Gets the value of the requested header @header if it exists, or
- * %NULL otherwise.
- *
- * Returns the value of the requested header (or %NULL if it isn't set)
- **/
-const char *
-g_mime_message_get_header (GMimeMessage *message, const char *header)
-{
-	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
-	g_return_val_if_fail (header != NULL, NULL);
-	
-	return g_mime_object_get_header (GMIME_OBJECT (message), header);
-}
-
-
 /**
  * g_mime_message_get_mime_part:
  * @message: MIME Message
  *
  * Gets the toplevel MIME part contained within @message.
  *
- * Returns the toplevel MIME part of @message.
+ * Returns: the toplevel MIME part of @message.
  **/
 GMimeObject *
 g_mime_message_get_mime_part (GMimeMessage *message)
@@ -1307,217 +1361,20 @@ void
 g_mime_message_set_mime_part (GMimeMessage *message, GMimeObject *mime_part)
 {
 	g_return_if_fail (GMIME_IS_MESSAGE (message));
+	g_return_if_fail (GMIME_IS_OBJECT (mime_part));
 	
 	g_object_ref (mime_part);
-	g_mime_header_set_raw (mime_part->headers, NULL);
+	g_mime_header_list_set_raw (mime_part->headers, NULL);
 	
-	if (message->mime_part)
+	if (message->mime_part) {
+		g_mime_header_list_set_raw (message->mime_part->headers, NULL);
 		g_object_unref (message->mime_part);
+	}
 	
 	message->mime_part = mime_part;
 }
 
 
-/**
- * g_mime_message_write_to_stream:
- * @message: MIME Message
- * @stream: output stream
- *
- * Write the contents of the MIME Message to @stream.
- *
- * WARNING: This interface is deprecated. Use
- * g_mime_object_write_to_stream() instead.
- *
- * Returns -1 on fail.
- **/
-ssize_t
-g_mime_message_write_to_stream (GMimeMessage *message, GMimeStream *stream)
-{
-	g_return_val_if_fail (GMIME_IS_MESSAGE (message), -1);
-	g_return_val_if_fail (GMIME_IS_STREAM (stream), -1);
-	
-	return g_mime_object_write_to_stream ((GMimeObject *) message, stream);
-}
-
-
-/**
- * g_mime_message_to_string:
- * @message: MIME Message
- *
- * Allocates a string buffer containing the mime message @message.
- *
- * WARNING: This interface is deprecated. Use
- * g_mime_object_to_string() instead.
- *
- * Returns an allocated string containing the MIME Message.
- **/
-char *
-g_mime_message_to_string (GMimeMessage *message)
-{
-	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
-	
-	return g_mime_object_to_string ((GMimeObject *) message);
-}
-
-
-/* The proper way to handle a multipart/alternative part is to return
- * the last part that we know how to render. For our purposes, we are
- * going to assume:
- *
- * If @want_plain is %FALSE then assume we can render text/plain and
- * text/html, thus the order of preference is text/html, then
- * text/plain and finally text/<any>.
- *
- * Otherwise, if @want_plain is %TRUE then we assume that we do not
- * know how to render text/html and so our order of preference becomes
- * text/plain and then text/<any>.
- **/
-static GMimeObject *
-handle_multipart_alternative (GMimeMultipart *multipart, gboolean want_plain, gboolean *is_html)
-{
-	GMimeObject *mime_part, *text_part = NULL;
-	const GMimeContentType *type;
-	GList *subpart;
-	
-	subpart = multipart->subparts;
-	while (subpart) {
-		mime_part = subpart->data;
-		
-		type = g_mime_object_get_content_type (mime_part);
-		if (g_mime_content_type_is_type (type, "text", "*")) {
-			if (!text_part || !g_ascii_strcasecmp (type->subtype, want_plain ? "plain" : "html")) {
-				*is_html = !g_ascii_strcasecmp (type->subtype, "html");
-				text_part = mime_part;
-			}
-		}
-		
-		subpart = subpart->next;
-	}
-	
-	return text_part;
-}
-
-static GMimeObject *
-handle_multipart_mixed (GMimeMultipart *multipart, gboolean want_plain, gboolean *is_html)
-{
-	GMimeObject *mime_part, *text_part = NULL;
-	const GMimeContentType *type, *first_type = NULL;
-	GList *subpart;
-	
-	subpart = multipart->subparts;
-	while (subpart) {
-		mime_part = subpart->data;
-		
-		type = g_mime_object_get_content_type (mime_part);
-		if (GMIME_IS_MULTIPART (mime_part)) {
-			multipart = GMIME_MULTIPART (mime_part);
-			if (g_mime_content_type_is_type (type, "multipart", "alternative")) {
-				mime_part = handle_multipart_alternative (multipart, want_plain, is_html);
-				if (mime_part)
-					return mime_part;
-			} else {
-				mime_part = handle_multipart_mixed (multipart, want_plain, is_html);
-				if (mime_part && !text_part)
-					text_part = mime_part;
-			}
-		} else if (g_mime_content_type_is_type (type, "text", "*")) {
-			if (!g_ascii_strcasecmp (type->subtype, want_plain ? "plain" : "html")) {
-				/* we got what we came for */
-				*is_html = !g_ascii_strcasecmp (type->subtype, "html");
-				return mime_part;
-			}
-			
-			/* if we haven't yet found a text part or if it is a type we can
-			 * understand and it is the first of that type, save it */
-			if (!text_part || (!g_ascii_strcasecmp (type->subtype, "plain") && (first_type &&
-					   g_ascii_strcasecmp (type->subtype, first_type->subtype) != 0))) {
-				*is_html = !g_ascii_strcasecmp (type->subtype, "html");
-				text_part = mime_part;
-				first_type = type;
-			}
-		}
-		
-		subpart = subpart->next;
-	}
-	
-	return text_part;
-}
-
-
-/**
- * g_mime_message_get_body:
- * @message: MIME Message
- * @want_plain: request text/plain
- * @is_html: body returned is in html format
- *
- * Attempts to get the body of the message in the preferred format
- * specified by @want_plain.
- *
- * Returns the prefered form of the message body. Sets the value of
- * @is_html to %TRUE if the part returned is in HTML format, otherwise
- * %FALSE.
- *
- * Note: This function is NOT guarenteed to always work as it
- * makes some assumptions that are not necessarily true. It is
- * recommended that you traverse the MIME structure yourself.
- **/
-char *
-g_mime_message_get_body (const GMimeMessage *message, gboolean want_plain, gboolean *is_html)
-{
-	GMimeObject *mime_part = NULL;
-	const GMimeContentType *type;
-	GMimeMultipart *multipart;
-	const char *content;
-	char *body = NULL;
-	size_t len = 0;
-	
-	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
-	g_return_val_if_fail (is_html != NULL, NULL);
-	
-	type = g_mime_object_get_content_type (message->mime_part);
-	if (GMIME_IS_MULTIPART (message->mime_part)) {
-		/* let's see if we can find a body in the multipart */
-		multipart = GMIME_MULTIPART (message->mime_part);
-		if (g_mime_content_type_is_type (type, "multipart", "alternative"))
-			mime_part = handle_multipart_alternative (multipart, want_plain, is_html);
-		else
-			mime_part = handle_multipart_mixed (multipart, want_plain, is_html);
-	} else if (g_mime_content_type_is_type (type, "text", "*")) {
-		/* this *has* to be the message body */
-		if (g_mime_content_type_is_type (type, "text", "html"))
-			*is_html = TRUE;
-		else
-			*is_html = FALSE;
-		
-		mime_part = message->mime_part;
-	}
-	
-	if (mime_part != NULL) {
-		content = g_mime_part_get_content (GMIME_PART (mime_part), &len);
-		body = g_strndup (content, len);
-	}
-	
-	return body;
-}
-
-
-/**
- * g_mime_message_get_headers:
- * @message: MIME Message
- *
- * Allocates a string buffer containing the raw message headers.
- *
- * Returns an allocated string containing the raw message headers.
- **/
-char *
-g_mime_message_get_headers (GMimeMessage *message)
-{
-	g_return_val_if_fail (GMIME_IS_MESSAGE (message), NULL);
-	
-	return g_mime_object_get_headers (GMIME_OBJECT (message));
-}
-
-
 /**
  * g_mime_message_foreach_part:
  * @message: MIME message
