@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2007 Jeffrey Stedfast
+ *  Copyright (C) 2000-2008 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -35,10 +35,10 @@
 /**
  * SECTION: gmime-object
  * @title: GMimeObject
- * @short_description: Abstract MIME objects
+ * @short_description: Abstract Mime objects
  * @see_also:
  *
- * #GMimeObject is an abstract class from which all Message and MIME
+ * #GMimeObject is an abstract class from which all Message and Mime
  * parts are derived.
  **/
 
@@ -59,15 +59,17 @@ static void g_mime_object_init (GMimeObject *object, GMimeObjectClass *klass);
 static void g_mime_object_finalize (GObject *object);
 
 static void init (GMimeObject *object);
-static void add_header (GMimeObject *object, const char *header, const char *value);
-static void set_header (GMimeObject *object, const char *header, const char *value);
-static const char *get_header (GMimeObject *object, const char *header);
-static void remove_header (GMimeObject *object, const char *header);
+static void prepend_header (GMimeObject *object, const char *name, const char *value);
+static void append_header (GMimeObject *object, const char *name, const char *value);
+static void set_header (GMimeObject *object, const char *name, const char *value);
+static const char *get_header (GMimeObject *object, const char *name);
+static gboolean remove_header (GMimeObject *object, const char *name);
 static void set_content_type (GMimeObject *object, GMimeContentType *content_type);
 static char *get_headers (GMimeObject *object);
 static ssize_t write_to_stream (GMimeObject *object, GMimeStream *stream);
 
 static ssize_t write_content_type (GMimeStream *stream, const char *name, const char *value);
+static ssize_t write_disposition (GMimeStream *stream, const char *name, const char *value);
 
 static void type_registry_init (void);
 
@@ -112,10 +114,11 @@ g_mime_object_class_init (GMimeObjectClass *klass)
 	object_class->finalize = g_mime_object_finalize;
 	
 	klass->init = init;
-	klass->add_header = add_header;
+	klass->prepend_header = prepend_header;
+	klass->append_header = append_header;
+	klass->remove_header = remove_header;
 	klass->set_header = set_header;
 	klass->get_header = get_header;
-	klass->remove_header = remove_header;
 	klass->set_content_type = set_content_type;
 	klass->get_headers = get_headers;
 	klass->write_to_stream = write_to_stream;
@@ -126,11 +129,13 @@ g_mime_object_class_init (GMimeObjectClass *klass)
 static void
 g_mime_object_init (GMimeObject *object, GMimeObjectClass *klass)
 {
+	object->headers = g_mime_header_list_new ();
 	object->content_type = NULL;
-	object->headers = g_mime_header_new ();
+	object->disposition = NULL;
 	object->content_id = NULL;
 	
-	g_mime_header_register_writer (object->headers, "Content-Type", write_content_type);
+	g_mime_header_list_register_writer (object->headers, "Content-Type", write_content_type);
+	g_mime_header_list_register_writer (object->headers, "Content-Disposition", write_disposition);
 }
 
 static void
@@ -141,8 +146,11 @@ g_mime_object_finalize (GObject *object)
 	if (mime->content_type)
 		g_mime_content_type_destroy (mime->content_type);
 	
+	if (mime->disposition)
+		g_mime_content_disposition_destroy (mime->disposition);
+	
 	if (mime->headers)
-		g_mime_header_destroy (mime->headers);
+		g_mime_header_list_destroy (mime->headers);
 	
 	g_free (mime->content_id);
 	
@@ -176,38 +184,67 @@ write_content_type (GMimeStream *stream, const char *name, const char *value)
 	return nwritten;
 }
 
-
-/**
- * g_mime_object_ref:
- * @object: mime object
- *
- * Ref's a MIME object.
- *
- * WARNING: This method is deprecated. Use g_object_ref() instead.
- **/
 void
-g_mime_object_ref (GMimeObject *object)
+_g_mime_object_content_type_changed (GMimeObject *object)
 {
-	g_return_if_fail (GMIME_IS_OBJECT (object));
+	GMimeContentType *content_type;
+	GMimeParam *params;
+	GString *string;
+	char *type, *p;
 	
-	g_object_ref (object);
+	content_type = object->content_type;
+	
+	string = g_string_new ("Content-Type: ");
+	
+	type = g_mime_content_type_to_string (content_type);
+	g_string_append (string, type);
+	g_free (type);
+	
+	if ((params = content_type->params))
+		g_mime_param_write_to_string (params, FALSE, string);
+	
+	p = string->str;
+	g_string_free (string, FALSE);
+	
+	type = p + strlen ("Content-Type: ");
+	g_mime_header_list_set (object->headers, "Content-Type", type);
+	g_free (p);
 }
 
-
-/**
- * g_mime_object_unref:
- * @object: mime object
- *
- * Unref's a MIME object.
- *
- * WARNING: This method is deprecated. Use g_object_unref() instead.
- **/
-void
-g_mime_object_unref (GMimeObject *object)
+static ssize_t
+write_disposition (GMimeStream *stream, const char *name, const char *value)
 {
-	g_return_if_fail (GMIME_IS_OBJECT (object));
+	GMimeContentDisposition *disposition;
+	ssize_t nwritten;
+	GString *out;
 	
-	g_object_unref (object);
+	out = g_string_new ("");
+	g_string_printf (out, "%s: ", name);
+	
+	disposition = g_mime_content_disposition_new_from_string (value);
+	g_string_append (out, disposition->disposition);
+	
+	g_mime_param_write_to_string (disposition->params, TRUE, out);
+	g_mime_content_disposition_destroy (disposition);
+	
+	nwritten = g_mime_stream_write (stream, out->str, out->len);
+	g_string_free (out, TRUE);
+	
+	return nwritten;
+}
+
+void
+_g_mime_object_content_disposition_changed (GMimeObject *object)
+{
+	char *str;
+	
+	if (object->disposition) {
+		str = g_mime_content_disposition_to_string (object->disposition, FALSE);
+		g_mime_header_list_set (object->headers, "Content-Disposition", str);
+		g_free (str);
+	} else {
+		g_mime_header_list_remove (object->headers, "Content-Disposition");
+	}
 }
 
 
@@ -264,12 +301,12 @@ init (GMimeObject *object)
  *
  * Performs a lookup of registered #GMimeObject subclasses, registered
  * using g_mime_object_register_type(), to find an appropriate class
- * capable of handling MIME parts of type @type/@subtype. If no class
+ * capable of handling Mime parts of type @type/@subtype. If no class
  * has been registered to handle that type, it looks for a registered
  * class that can handle @type. If that also fails, then it will use
  * the generic part class, #GMimePart.
  *
- * Returns an appropriate #GMimeObject registered to handle mime-types
+ * Returns: an appropriate #GMimeObject registered to handle mime-types
  * of @type/@subtype.
  **/
 GMimeObject *
@@ -312,53 +349,25 @@ g_mime_object_new_type (const char *type, const char *subtype)
 	return object;
 }
 
-
-static void
-sync_content_type (GMimeObject *object)
-{
-	GMimeContentType *content_type;
-	GMimeParam *params;
-	GString *string;
-	char *type, *p;
-	
-	content_type = object->content_type;
-	
-	string = g_string_new ("Content-Type: ");
-	
-	type = g_mime_content_type_to_string (content_type);
-	g_string_append (string, type);
-	g_free (type);
-	
-	if ((params = content_type->params))
-		g_mime_param_write_to_string (params, FALSE, string);
-	
-	p = string->str;
-	g_string_free (string, FALSE);
-	
-	type = p + strlen ("Content-Type: ");
-	g_mime_header_set (object->headers, "Content-Type", type);
-	g_free (p);
-}
-
-
 static void
 set_content_type (GMimeObject *object, GMimeContentType *content_type)
 {
 	if (object->content_type)
 		g_mime_content_type_destroy (object->content_type);
 	
-	object->content_type = content_type;
+	if ((object->content_type = content_type))
+		content_type->parent_object = object;
 	
-	sync_content_type (object);
+	_g_mime_object_content_type_changed (object);
 }
 
 
 /**
  * g_mime_object_set_content_type:
- * @object: MIME object
- * @mime_type: MIME type
+ * @object: a #GMimeObject
+ * @mime_type: Mime type
  *
- * Sets the content-type for the specified MIME object.
+ * Sets the content-type for the specified Mime object.
  **/
 void
 g_mime_object_set_content_type (GMimeObject *object, GMimeContentType *mime_type)
@@ -372,12 +381,12 @@ g_mime_object_set_content_type (GMimeObject *object, GMimeContentType *mime_type
 
 /**
  * g_mime_object_get_content_type:
- * @object: MIME object
+ * @object: a #GMimeObject
  *
- * Gets the Content-Type object for the given MIME object or %NULL on
- * fail.
+ * Gets the #GMimeContentType object for the given Mime object or
+ * %NULL on fail.
  *
- * Returns the content-type object for the specified MIME object.
+ * Returns: the content-type object for the specified Mime object.
  **/
 const GMimeContentType *
 g_mime_object_get_content_type (GMimeObject *object)
@@ -390,7 +399,7 @@ g_mime_object_get_content_type (GMimeObject *object)
 
 /**
  * g_mime_object_set_content_type_parameter:
- * @object: MIME object
+ * @object: a #GMimeObject
  * @name: param name
  * @value: param value
  *
@@ -403,20 +412,18 @@ g_mime_object_set_content_type_parameter (GMimeObject *object, const char *name,
 	g_return_if_fail (name != NULL);
 	
 	g_mime_content_type_set_parameter (object->content_type, name, value);
-	
-	sync_content_type (object);
 }
 
 
 /**
  * g_mime_object_get_content_type_parameter:
- * @object: MIME object
+ * @object: a #GMimeObject
  * @name: param name
  *
- * Gets the value of the content-type param @name set on the MIME part
+ * Gets the value of the content-type param @name set on the Mime part
  * @object.
  *
- * Returns the value of the requested content-type param or %NULL on
+ * Returns: the value of the requested content-type param or %NULL on
  * if the param doesn't exist.
  **/
 const char *
@@ -429,12 +436,144 @@ g_mime_object_get_content_type_parameter (GMimeObject *object, const char *name)
 }
 
 
+/**
+ * g_mime_object_get_content_disposition:
+ * @object: a #GMimeObject
+ *
+ * Gets the #GMimeContentDisposition for the specified Mime object.
+ *
+ * Returns: the #GMimeContentDisposition set on the Mime object.
+ **/
+const GMimeContentDisposition *
+g_mime_object_get_content_disposition (GMimeObject *object)
+{
+	g_return_val_if_fail (GMIME_IS_OBJECT (object), NULL);
+	
+	return object->disposition;
+}
+
+
+/**
+ * g_mime_object_set_content_disposition:
+ * @object: a #GMimeObject
+ * @disposition: a #GMimeContentDisposition object
+ *
+ * Set the content disposition for the specified mime part.
+ **/
+void
+g_mime_object_set_content_disposition (GMimeObject *object, GMimeContentDisposition *disposition)
+{
+	g_return_if_fail (GMIME_IS_OBJECT (object));
+	
+	if (object->disposition)
+		g_mime_content_disposition_destroy (object->disposition);
+	
+	if ((object->disposition = disposition))
+		disposition->parent_object = object;
+	
+	_g_mime_object_content_disposition_changed (object);
+}
+
+
+/**
+ * g_mime_object_set_disposition:
+ * @object: a #GMimeObject
+ * @disposition: disposition ("attachment" or "inline")
+ *
+ * Sets the disposition to @disposition which may be one of
+ * #GMIME_DISPOSITION_ATTACHMENT or #GMIME_DISPOSITION_INLINE or, by
+ * your choice, any other string which would indicate how the Mime
+ * part should be displayed by the MUA.
+ **/
+void
+g_mime_object_set_disposition (GMimeObject *object, const char *disposition)
+{
+	g_return_if_fail (GMIME_IS_OBJECT (object));
+	
+	if (object->disposition) {
+		g_mime_content_disposition_set_disposition (object->disposition, disposition);
+	} else if (disposition) {
+		if ((object->disposition = g_mime_content_disposition_new_from_string (disposition)))
+			object->disposition->parent_object = object;
+		_g_mime_object_content_disposition_changed (object);
+	}
+}
+
+
+/**
+ * g_mime_object_get_disposition:
+ * @object: a #GMimeObject
+ *
+ * Gets the Mime object's disposition if set or %NULL otherwise.
+ *
+ * Returns: the disposition string which is probably one of
+ * #GMIME_DISPOSITION_ATTACHMENT or #GMIME_DISPOSITION_INLINE.
+ **/
+const char *
+g_mime_object_get_disposition (GMimeObject *object)
+{
+	g_return_val_if_fail (GMIME_IS_OBJECT (object), NULL);
+	
+	if (object->disposition)
+		return g_mime_content_disposition_get_disposition (object->disposition);
+	
+	return NULL;
+}
+
+
+/**
+ * g_mime_object_set_content_disposition_parameter:
+ * @object: a #GMimeObject
+ * @attribute: parameter name
+ * @value: parameter value
+ *
+ * Add a content-disposition parameter to the specified mime part.
+ **/
+void
+g_mime_object_set_content_disposition_parameter (GMimeObject *object, const char *attribute, const char *value)
+{
+	g_return_if_fail (GMIME_IS_OBJECT (object));
+	g_return_if_fail (attribute != NULL);
+	
+	if (!object->disposition) {
+		object->disposition = g_mime_content_disposition_new ();
+		object->disposition->parent_object = object;
+	}
+	
+	g_mime_content_disposition_set_parameter (object->disposition, attribute, value);
+}
+
+
+/**
+ * g_mime_object_get_content_disposition_parameter:
+ * @object: a #GMimeObject
+ * @attribute: parameter name
+ *
+ * Gets the value of the Content-Disposition parameter specified by
+ * @attribute, or %NULL if the parameter does not exist.
+ *
+ * Returns: the value of a previously defined content-disposition
+ * parameter specified by @attribute.
+ **/
+const char *
+g_mime_object_get_content_disposition_parameter (GMimeObject *object, const char *attribute)
+{
+	g_return_val_if_fail (GMIME_IS_OBJECT (object), NULL);
+	g_return_val_if_fail (attribute != NULL, NULL);
+	
+	if (!object->disposition)
+		return NULL;
+	
+	return g_mime_content_disposition_get_parameter (object->disposition, attribute);
+}
+
+
 /**
  * g_mime_object_set_content_id:
- * @object: MIME object
+ * @object: a #GMimeObject
  * @content_id: content-id (addr-spec portion)
  *
- * Sets the Content-Id of the MIME object.
+ * Sets the Content-Id of the Mime object.
  **/
 void
 g_mime_object_set_content_id (GMimeObject *object, const char *content_id)
@@ -454,11 +593,11 @@ g_mime_object_set_content_id (GMimeObject *object, const char *content_id)
 
 /**
  * g_mime_object_get_content_id:
- * @object: MIME object
+ * @object: a #GMimeObject
  *
- * Gets the Content-Id of the MIME object or NULL if one is not set.
+ * Gets the Content-Id of the Mime object or NULL if one is not set.
  *
- * Returns a const pointer to the Content-Id header.
+ * Returns: a const pointer to the Content-Id header.
  **/
 const char *
 g_mime_object_get_content_id (GMimeObject *object)
@@ -470,29 +609,37 @@ g_mime_object_get_content_id (GMimeObject *object)
 
 
 enum {
+	HEADER_CONTENT_DISPOSITION,
 	HEADER_CONTENT_TYPE,
 	HEADER_CONTENT_ID,
 	HEADER_UNKNOWN,
 };
 
-static char *headers[] = {
+static char *content_headers[] = {
+	"Content-Disposition",
 	"Content-Type",
 	"Content-Id",
-	NULL
 };
 
 static gboolean
 process_header (GMimeObject *object, const char *header, const char *value)
 {
+	GMimeContentDisposition *disposition;
 	GMimeContentType *content_type;
 	int i;
 	
-	for (i = 0; headers[i]; i++) {
-		if (!g_ascii_strcasecmp (headers[i], header))
+	for (i = 0; i < G_N_ELEMENTS (content_headers); i++) {
+		if (!g_ascii_strcasecmp (content_headers[i], header))
 			break;
 	}
 	
 	switch (i) {
+	case HEADER_CONTENT_DISPOSITION:
+		disposition = g_mime_content_disposition_new_from_string (value);
+		if (object->disposition)
+			g_mime_content_disposition_destroy (object->disposition);
+		object->disposition = disposition;
+		break;
 	case HEADER_CONTENT_TYPE:
 		content_type = g_mime_content_type_new_from_string (value);
 		g_mime_object_set_content_type (object, content_type);
@@ -505,35 +652,61 @@ process_header (GMimeObject *object, const char *header, const char *value)
 		return FALSE;
 	}
 	
-	g_mime_header_set (object->headers, header, value);
+	g_mime_header_list_set (object->headers, header, value);
 	
 	return TRUE;
 }
 
 static void
-add_header (GMimeObject *object, const char *header, const char *value)
+prepend_header (GMimeObject *object, const char *header, const char *value)
 {
 	if (!process_header (object, header, value))
-		g_mime_header_add (object->headers, header, value);
+		g_mime_header_list_prepend (object->headers, header, value);
 }
 
 
 /**
- * g_mime_object_add_header:
- * @object: mime object
+ * g_mime_object_prepend_header:
+ * @object: a #GMimeObject
  * @header: header name
  * @value: header value
  *
- * Adds an arbitrary header to the MIME object.
+ * Prepends a header to the Mime object.
  **/
 void
-g_mime_object_add_header (GMimeObject *object, const char *header, const char *value)
+g_mime_object_prepend_header (GMimeObject *object, const char *header, const char *value)
 {
 	g_return_if_fail (GMIME_IS_OBJECT (object));
 	g_return_if_fail (header != NULL);
 	g_return_if_fail (value != NULL);
 	
-	GMIME_OBJECT_GET_CLASS (object)->add_header (object, header, value);
+	GMIME_OBJECT_GET_CLASS (object)->prepend_header (object, header, value);
+}
+
+static void
+append_header (GMimeObject *object, const char *header, const char *value)
+{
+	if (!process_header (object, header, value))
+		g_mime_header_list_append (object->headers, header, value);
+}
+
+
+/**
+ * g_mime_object_append_header:
+ * @object: a #GMimeObject
+ * @header: header name
+ * @value: header value
+ *
+ * Appends a header to the Mime object.
+ **/
+void
+g_mime_object_append_header (GMimeObject *object, const char *header, const char *value)
+{
+	g_return_if_fail (GMIME_IS_OBJECT (object));
+	g_return_if_fail (header != NULL);
+	g_return_if_fail (value != NULL);
+	
+	GMIME_OBJECT_GET_CLASS (object)->append_header (object, header, value);
 }
 
 
@@ -541,17 +714,17 @@ static void
 set_header (GMimeObject *object, const char *header, const char *value)
 {
 	if (!process_header (object, header, value))
-		g_mime_header_set (object->headers, header, value);
+		g_mime_header_list_set (object->headers, header, value);
 }
 
 
 /**
  * g_mime_object_set_header:
- * @object: mime object
+ * @object: a #GMimeObject
  * @header: header name
  * @value: header value
  *
- * Sets an arbitrary header on the MIME object.
+ * Sets an arbitrary header on the Mime object.
  **/
 void
 g_mime_object_set_header (GMimeObject *object, const char *header, const char *value)
@@ -567,19 +740,19 @@ g_mime_object_set_header (GMimeObject *object, const char *header, const char *v
 static const char *
 get_header (GMimeObject *object, const char *header)
 {
-	return g_mime_header_get (object->headers, header);
+	return g_mime_header_list_get (object->headers, header);
 }
 
 
 /**
  * g_mime_object_get_header:
- * @object: mime object
+ * @object: a #GMimeObject
  * @header: header name
  *
  * Gets the value of the requested header if it exists or %NULL
  * otherwise.
  *
- * Returns the value of the header @header if it exists or %NULL
+ * Returns: the value of the header @header if it exists or %NULL
  * otherwise.
  **/
 const char *
@@ -592,45 +765,73 @@ g_mime_object_get_header (GMimeObject *object, const char *header)
 }
 
 
-static void
+static gboolean
 remove_header (GMimeObject *object, const char *header)
 {
-	g_mime_header_remove (object->headers, header);
+	int i;
+	
+	for (i = 0; i < G_N_ELEMENTS (content_headers); i++) {
+		if (!g_ascii_strcasecmp (content_headers[i], header))
+			break;
+	}
+	
+	switch (i) {
+	case HEADER_CONTENT_DISPOSITION:
+		if (object->disposition) {
+			g_mime_content_disposition_destroy (object->disposition);
+			object->disposition = NULL;
+		}
+		break;
+	case HEADER_CONTENT_TYPE:
+		/* never allow the removal of the Content-Type header */
+		return FALSE;
+	case HEADER_CONTENT_ID:
+		g_free (object->content_id);
+		object->content_id = NULL;
+		break;
+	default:
+		break;
+	}
+	
+	return g_mime_header_list_remove (object->headers, header);
 }
 
 
 /**
  * g_mime_object_remove_header:
- * @object: mime object
+ * @object: a #GMimeObject
  * @header: header name
  *
  * Removed the specified header if it exists.
+ *
+ * Returns: %TRUE if the header was removed or %FALSE if it could not
+ * be found.
  **/
-void
+gboolean
 g_mime_object_remove_header (GMimeObject *object, const char *header)
 {
 	g_return_if_fail (GMIME_IS_OBJECT (object));
 	g_return_if_fail (header != NULL);
 	
-	GMIME_OBJECT_GET_CLASS (object)->remove_header (object, header);
+	return GMIME_OBJECT_GET_CLASS (object)->remove_header (object, header);
 }
 
 
 static char *
 get_headers (GMimeObject *object)
 {
-	return g_mime_header_to_string (object->headers);
+	return g_mime_header_list_to_string (object->headers);
 }
 
 
 /**
  * g_mime_object_get_headers:
- * @object: mime object
+ * @object: a #GMimeObject
  *
- * Allocates a string buffer containing all of the MIME object's raw
+ * Allocates a string buffer containing all of the Mime object's raw
  * headers.
  *
- * Returns an allocated string containing all of the raw MIME headers.
+ * Returns: an allocated string containing all of the raw Mime headers.
  **/
 char *
 g_mime_object_get_headers (GMimeObject *object)
@@ -650,12 +851,12 @@ write_to_stream (GMimeObject *object, GMimeStream *stream)
 
 /**
  * g_mime_object_write_to_stream:
- * @object: mime object
+ * @object: a #GMimeObject
  * @stream: stream
  *
- * Write the contents of the MIME object to @stream.
+ * Write the contents of the Mime object to @stream.
  *
- * Returns -1 on fail.
+ * Returns: the number of bytes written or %-1 on fail.
  **/
 ssize_t
 g_mime_object_write_to_stream (GMimeObject *object, GMimeStream *stream)
@@ -669,11 +870,11 @@ g_mime_object_write_to_stream (GMimeObject *object, GMimeStream *stream)
 
 /**
  * g_mime_object_to_string:
- * @object: mime object
+ * @object: a #GMimeObject
  *
  * Allocates a string buffer containing the contents of @object.
  *
- * Returns an allocated string containing the contents of the mime
+ * Returns: an allocated string containing the contents of the mime
  * object.
  **/
 char *
@@ -700,6 +901,24 @@ g_mime_object_to_string (GMimeObject *object)
 }
 
 
+/**
+ * g_mime_object_get_header_list:
+ * @object: a #GMimeObject
+ *
+ * Get the header list for @object.
+ *
+ * Returns: the #GMimeHeaderList for @object. Do not free this pointer
+ * when you are done with it.
+ **/
+GMimeHeaderList *
+g_mime_object_get_header_list (GMimeObject *object)
+{
+	g_return_val_if_fail (GMIME_IS_OBJECT (object), NULL);
+	
+	return object->headers;
+}
+
+
 static void
 subtype_bucket_foreach (gpointer key, gpointer value, gpointer user_data)
 {
