@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2007 Jeffrey Stedfast
+ *  Copyright (C) 2000-2008 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -26,6 +26,7 @@
 #include <glib-object.h>
 
 #include <gmime/gmime-content-type.h>
+#include <gmime/gmime-disposition.h>
 #include <gmime/gmime-stream.h>
 #include <gmime/gmime-header.h>
 
@@ -44,8 +45,9 @@ typedef struct _GMimeObjectClass GMimeObjectClass;
 struct _GMimeObject {
 	GObject parent_object;
 	
+	GMimeContentDisposition *disposition;
 	GMimeContentType *content_type;
-	GMimeHeader *headers;
+	GMimeHeaderList *headers;
 	
 	char *content_id;
 };
@@ -55,10 +57,11 @@ struct _GMimeObjectClass {
 	
 	void         (* init)          (GMimeObject *object);
 	
-	void         (* add_header)    (GMimeObject *object, const char *header, const char *value);
-	void         (* set_header)    (GMimeObject *object, const char *header, const char *value);
-	const char * (* get_header)    (GMimeObject *object, const char *header);
-	void         (* remove_header) (GMimeObject *object, const char *header);
+	void         (* prepend_header) (GMimeObject *object, const char *header, const char *value);
+	void         (* append_header)  (GMimeObject *object, const char *header, const char *value);
+	void         (* set_header)     (GMimeObject *object, const char *header, const char *value);
+	const char * (* get_header)     (GMimeObject *object, const char *header);
+	gboolean     (* remove_header)  (GMimeObject *object, const char *header);
 	
 	void         (* set_content_type) (GMimeObject *object, GMimeContentType *content_type);
 	
@@ -84,24 +87,30 @@ GType g_mime_object_get_type (void);
 void g_mime_object_register_type (const char *type, const char *subtype, GType object_type);
 GMimeObject *g_mime_object_new_type (const char *type, const char *subtype);
 
-#ifndef GMIME_DISABLE_DEPRECATED
-void g_mime_object_ref (GMimeObject *object);
-void g_mime_object_unref (GMimeObject *object);
-#endif
-
 void g_mime_object_set_content_type (GMimeObject *object, GMimeContentType *mime_type);
 const GMimeContentType *g_mime_object_get_content_type (GMimeObject *object);
-
 void g_mime_object_set_content_type_parameter (GMimeObject *object, const char *name, const char *value);
 const char *g_mime_object_get_content_type_parameter (GMimeObject *object, const char *name);
 
+void g_mime_object_set_content_disposition (GMimeObject *object, GMimeContentDisposition *disposition);
+const GMimeContentDisposition *g_mime_object_get_content_disposition (GMimeObject *object);
+
+void g_mime_object_set_disposition (GMimeObject *object, const char *disposition);
+const char *g_mime_object_get_disposition (GMimeObject *object);
+
+void g_mime_object_set_content_disposition_parameter (GMimeObject *object, const char *attribute, const char *value);
+const char *g_mime_object_get_content_disposition_parameter (GMimeObject *object, const char *attribute);
+
 void g_mime_object_set_content_id (GMimeObject *object, const char *content_id);
 const char *g_mime_object_get_content_id (GMimeObject *object);
 
-void g_mime_object_add_header (GMimeObject *object, const char *header, const char *value);
+void g_mime_object_prepend_header (GMimeObject *object, const char *header, const char *value);
+void g_mime_object_append_header (GMimeObject *object, const char *header, const char *value);
 void g_mime_object_set_header (GMimeObject *object, const char *header, const char *value);
 const char *g_mime_object_get_header (GMimeObject *object, const char *header);
-void g_mime_object_remove_header (GMimeObject *object, const char *header);
+gboolean g_mime_object_remove_header (GMimeObject *object, const char *header);
+
+GMimeHeaderList *g_mime_object_get_header_list (GMimeObject *object);
 
 char *g_mime_object_get_headers (GMimeObject *object);
 
