@@ -87,7 +87,7 @@ enum {
 struct _GMimeParserPrivate {
 	GMimeStream *stream;
 	
-	off_t offset;
+	gint64 offset;
 	
 	/* i/o buffers */
 	char realbuf[SCAN_HEAD + SCAN_BUF + 1];
@@ -95,7 +95,7 @@ struct _GMimeParserPrivate {
 	char *inptr;
 	char *inend;
 	
-	off_t from_offset;
+	gint64 from_offset;
 	GByteArray *from_line;
 	
 	regex_t header_regex;
@@ -105,23 +105,25 @@ struct _GMimeParserPrivate {
 	/* header buffer */
 	char *headerbuf;
 	char *headerptr;
-	guint headerleft;
+	size_t headerleft;
 	
 	/* raw header buffer */
 	char *rawbuf;
 	char *rawptr;
-	guint rawleft;
+	size_t rawleft;
 	
-	off_t headers_start;
-	off_t header_start;
+	gint64 headers_start;
+	gint64 header_start;
 	
-	int state:26;
-	unsigned int midline:1;
-	unsigned int seekable:1;
-	unsigned int scan_from:1;
-	unsigned int have_regex:1;
-	unsigned int persist_stream:1;
-	unsigned int respect_content_length:1;
+	short int state;
+	
+	unsigned short int unused:10;
+	unsigned short int midline:1;
+	unsigned short int seekable:1;
+	unsigned short int scan_from:1;
+	unsigned short int have_regex:1;
+	unsigned short int persist_stream:1;
+	unsigned short int respect_content_length:1;
 	
 	GMimeContentType *content_type;
 	struct _header_raw *headers;
@@ -135,7 +137,7 @@ struct _boundary_stack {
 	size_t boundarylen;
 	size_t boundarylenfinal;
 	size_t boundarylenmax;
-	off_t content_end;
+	gint64 content_end;
 };
 
 
@@ -189,11 +191,11 @@ struct _header_raw {
 	struct _header_raw *next;
 	char *name;
 	char *value;
-	off_t offset;
+	gint64 offset;
 };
 
 static const char *
-header_raw_find (struct _header_raw *headers, const char *name, off_t *offset)
+header_raw_find (struct _header_raw *headers, const char *name, gint64 *offset)
 {
 	struct _header_raw *h;
 	
@@ -295,7 +297,7 @@ static void
 parser_init (GMimeParser *parser, GMimeStream *stream)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
-	off_t offset = -1;
+	gint64 offset = -1;
 	
 	if (stream) {
 		g_object_ref (stream);
@@ -359,7 +361,7 @@ parser_close (GMimeParser *parser)
  *
  * Creates a new parser object.
  *
- * Returns a new parser object.
+ * Returns: a new parser object.
  **/
 GMimeParser *
 g_mime_parser_new (void)
@@ -378,7 +380,7 @@ g_mime_parser_new (void)
  *
  * Creates a new parser object preset to parse @stream.
  *
- * Returns a new parser object.
+ * Returns: a new parser object.
  **/
 GMimeParser *
 g_mime_parser_new_with_stream (GMimeStream *stream)
@@ -430,7 +432,7 @@ g_mime_parser_init_with_stream (GMimeParser *parser, GMimeStream *stream)
  *
  * Gets whether or not the underlying stream is persistent.
  *
- * Returns %TRUE if the @parser will leave the content on disk or
+ * Returns: %TRUE if the @parser will leave the content on disk or
  * %FALSE if it will load the content into memory.
  **/
 gboolean
@@ -475,7 +477,7 @@ g_mime_parser_set_persist_stream (GMimeParser *parser, gboolean persist)
  *
  * Gets whether or not @parser is set to scan mbox-style From-lines.
  *
- * Returns whether or not @parser is set to scan mbox-style
+ * Returns: whether or not @parser is set to scan mbox-style
  * From-lines.
  **/
 gboolean
@@ -510,7 +512,7 @@ g_mime_parser_set_scan_from (GMimeParser *parser, gboolean scan_from)
  * Gets whether or not @parser is set to use Content-Length for
  * determining the offset of the end of the message.
  *
- * Returns whether or not @parser is set to use Content-Length for
+ * Returns: whether or not @parser is set to use Content-Length for
  * determining the offset of the end of the message.
  **/
 gboolean
@@ -625,7 +627,7 @@ parser_fill (GMimeParser *parser)
 }
 
 
-static off_t
+static gint64
 parser_offset (struct _GMimeParserPrivate *priv, const char *inptr)
 {
 	if (priv->offset == -1)
@@ -644,10 +646,10 @@ parser_offset (struct _GMimeParserPrivate *priv, const char *inptr)
  *
  * Gets the current stream offset from the parser's internal stream.
  *
- * Returns the current stream offset from the parser's internal stream
+ * Returns: the current stream offset from the parser's internal stream
  * or -1 on error.
  **/
-off_t
+gint64
 g_mime_parser_tell (GMimeParser *parser)
 {
 	g_return_val_if_fail (GMIME_IS_PARSER (parser), -1);
@@ -663,7 +665,7 @@ g_mime_parser_tell (GMimeParser *parser)
  *
  * Tests the end-of-stream indicator for @parser's internal stream.
  *
- * Returns %TRUE on EOS or %FALSE otherwise.
+ * Returns: %TRUE on EOS or %FALSE otherwise.
  **/
 gboolean
 g_mime_parser_eos (GMimeParser *parser)
@@ -793,9 +795,6 @@ header_parse (GMimeParser *parser, struct _header_raw **tail)
 	char *start;
 	size_t hlen;
 	
-	header = g_new (struct _header_raw, 1);
-	header->next = NULL;
-	
 	*priv->headerptr = '\0';
 	inptr = priv->headerbuf;
 	while (*inptr && *inptr != ':' && !is_type (*inptr, IS_SPACE | IS_CTRL))
@@ -814,6 +813,10 @@ header_parse (GMimeParser *parser, struct _header_raw **tail)
 	}
 	
 	hlen = inptr - priv->headerbuf;
+	
+	header = g_new (struct _header_raw, 1);
+	header->next = NULL;
+	
 	header->name = g_strndup (priv->headerbuf, hlen);
 	
 	/* skip over leading lwsp */
@@ -850,7 +853,8 @@ static int
 parser_step_headers (GMimeParser *parser)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
-	gboolean valid = TRUE, fieldname = TRUE;
+	gboolean eoln, valid = TRUE, fieldname = TRUE;
+	gboolean continuation = FALSE;
 	struct _header_raw *hend;
 	register char *inptr;
 	char *start, *inend;
@@ -882,10 +886,22 @@ parser_step_headers (GMimeParser *parser)
 		while (inptr < inend) {
 			start = inptr;
 			
-			if (fieldname && *inptr != '\n') {
+			/* if we are scanning a new line, check for a folded header */
+			if (!priv->midline && continuation && (*inptr != ' ' && *inptr != '\t')) {
+				header_parse (parser, &hend);
+				priv->header_start = parser_offset (priv, inptr);
+				continuation = FALSE;
+				fieldname = TRUE;
+				valid = TRUE;
+			}
+			
+			eoln = inptr[0] == '\n' || (inptr[0] == '\r' && inptr[1] == '\n');
+			if (fieldname && !eoln) {
 				/* scan and validate the field name */
 				if (*inptr != ':') {
+					/* Note: see optimization comment [1] */
 					*inend = ':';
+					
 					while (*inptr != ':') {
 						if (is_type (*inptr, IS_SPACE | IS_CTRL)) {
 							valid = FALSE;
@@ -902,6 +918,7 @@ parser_step_headers (GMimeParser *parser)
 						goto refill;
 					}
 					
+					/* Note: see optimization comment [1] */
 					*inend = '\n';
 				} else if (*inptr == ':') {
 					valid = FALSE;
@@ -928,11 +945,17 @@ parser_step_headers (GMimeParser *parser)
 				inptr++;
 			
 			len = (size_t) (inptr - start);
-			raw_header_append (priv, start, len);
 			
 			if (inptr == inend) {
-				/* we don't have enough data to tell if we
-				   got all of the header or not... */
+				/* we don't have the full line, save
+				 * what we have and refill our
+				 * buffer... */
+				if (inptr > start && inptr[-1] == '\r') {
+					inptr--;
+					len--;
+				}
+				
+				raw_header_append (priv, start, len);
 				header_append (priv, start, len);
 				priv->midline = TRUE;
 				left = inend - inptr;
@@ -940,28 +963,22 @@ parser_step_headers (GMimeParser *parser)
 				goto refill;
 			}
 			
-			/* check to see if we've reached the end of the headers */
-			if (!priv->midline && (inptr == start || (len == 1 && *start == '\r')))
-				goto headers_end;
+			raw_header_append (priv, start, len);
 			
 			if (inptr > start && inptr[-1] == '\r')
 				len--;
 			
+			/* check to see if we've reached the end of the headers */
+			if (!priv->midline && len == 0)
+				goto headers_end;
+			
 			header_append (priv, start, len);
 			
 			/* inptr has to be less than inend - 1 */
-			raw_header_append (priv, inptr, 1);
+			raw_header_append (priv, "\n", 1);
+			priv->midline = FALSE;
+			continuation = TRUE;
 			inptr++;
-			
-			if (*inptr == ' ' || *inptr == '\t') {
-				priv->midline = TRUE;
-			} else {
-				header_parse (parser, &hend);
-				priv->header_start = parser_offset (priv, inptr);
-				priv->midline = FALSE;
-				fieldname = TRUE;
-				valid = TRUE;
-			}
 		}
 		
 		left = inend - inptr;
@@ -1108,7 +1125,7 @@ enum {
 static int
 check_boundary (struct _GMimeParserPrivate *priv, const char *start, size_t len)
 {
-	off_t offset = parser_offset (priv, start);
+	gint64 offset = parser_offset (priv, start);
 	
 	if (start[len - 1] == '\r')
 		len--;
@@ -1250,11 +1267,11 @@ static void
 parser_scan_mime_part_content (GMimeParser *parser, GMimePart *mime_part, int *found)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
-	GMimePartEncodingType encoding;
+	GMimeContentEncoding encoding;
 	GByteArray *content = NULL;
 	GMimeDataWrapper *wrapper;
 	GMimeStream *stream;
-	off_t start, end;
+	gint64 start, end;
 	int crlf;
 	
 	g_assert (priv->state >= GMIME_PARSER_STATE_HEADERS_END);
@@ -1277,7 +1294,7 @@ parser_scan_mime_part_content (GMimeParser *parser, GMimePart *mime_part, int *f
 		end = parser_offset (priv, NULL);
 	}
 	
-	encoding = g_mime_part_get_encoding (mime_part);
+	encoding = g_mime_part_get_content_encoding (mime_part);
 	
 	if (priv->persist_stream && priv->seekable)
 		stream = g_mime_stream_substream (priv->stream, start, end);
@@ -1303,7 +1320,7 @@ parser_scan_message_part (GMimeParser *parser, GMimeMessagePart *mpart, int *fou
 	
 	/* get the headers */
 	priv->state = GMIME_PARSER_STATE_HEADERS;
-	if (parser_step (parser) == -1) {
+	if (parser_step (parser) == GMIME_PARSER_STATE_ERROR) {
 		/* Note: currently cannot happen because
 		 * parser_step_headers() never returns error */
 		*found = FOUND_EOS;
@@ -1313,7 +1330,7 @@ parser_scan_message_part (GMimeParser *parser, GMimeMessagePart *mpart, int *fou
 	message = g_mime_message_new (FALSE);
 	header = priv->headers;
 	while (header) {
-		g_mime_object_add_header ((GMimeObject *) message, header->name, header->value);
+		g_mime_object_append_header ((GMimeObject *) message, header->name, header->value);
 		header = header->next;
 	}
 	
@@ -1344,7 +1361,7 @@ parser_construct_leaf_part (GMimeParser *parser, GMimeContentType *content_type,
 	
 	header = priv->headers;
 	while (header) {
-		g_mime_object_add_header (object, header->name, header->value);
+		g_mime_object_append_header (object, header->name, header->value);
 		header = header->next;
 	}
 	
@@ -1354,12 +1371,12 @@ parser_construct_leaf_part (GMimeParser *parser, GMimeContentType *content_type,
 		g_mime_content_type_destroy (object->content_type);
 	object->content_type = content_type;
 	
-	g_mime_header_set_raw (object->headers, priv->rawbuf);
+	g_mime_header_list_set_raw (object->headers, priv->rawbuf);
 	raw_header_reset (priv);
 	
 	if (priv->state == GMIME_PARSER_STATE_HEADERS_END) {
 		/* skip empty line after headers */
-		if (parser_step (parser) == -1) {
+		if (parser_step (parser) == GMIME_PARSER_STATE_ERROR) {
 			*found = FOUND_EOS;
 			return object;
 		}
@@ -1456,7 +1473,7 @@ parser_scan_multipart_subparts (GMimeParser *parser, GMimeMultipart *multipart)
 		
 		/* get the headers */
 		priv->state = GMIME_PARSER_STATE_HEADERS;
-		if (parser_step (parser) == -1) {
+		if (parser_step (parser) == GMIME_PARSER_STATE_ERROR) {
 			found = FOUND_EOS;
 			break;
 		}
@@ -1496,7 +1513,7 @@ parser_construct_multipart (GMimeParser *parser, GMimeContentType *content_type,
 	
 	header = priv->headers;
 	while (header) {
-		g_mime_object_add_header (object, header->name, header->value);
+		g_mime_object_append_header (object, header->name, header->value);
 		header = header->next;
 	}
 	
@@ -1506,14 +1523,14 @@ parser_construct_multipart (GMimeParser *parser, GMimeContentType *content_type,
 		g_mime_content_type_destroy (object->content_type);
 	object->content_type = content_type;
 	
-	g_mime_header_set_raw (object->headers, priv->rawbuf);
+	g_mime_header_list_set_raw (object->headers, priv->rawbuf);
 	raw_header_reset (priv);
 	
 	multipart = (GMimeMultipart *) object;
 	
 	if (priv->state == GMIME_PARSER_STATE_HEADERS_END) {
 		/* skip empty line after headers */
-		if (parser_step (parser) == -1) {
+		if (parser_step (parser) == GMIME_PARSER_STATE_ERROR) {
 			*found = FOUND_EOS;
 			return object;
 		}
@@ -1555,7 +1572,7 @@ parser_construct_part (GMimeParser *parser)
 	
 	/* get the headers */
 	while (priv->state < GMIME_PARSER_STATE_HEADERS_END) {
-		if (parser_step (parser) == -1)
+		if (parser_step (parser) == GMIME_PARSER_STATE_ERROR)
 			return NULL;
 	}
 	
@@ -1577,7 +1594,7 @@ parser_construct_part (GMimeParser *parser)
  *
  * Constructs a MIME part from @parser.
  *
- * Returns a MIME part based on @parser or %NULL on fail.
+ * Returns: a MIME part based on @parser or %NULL on fail.
  **/
 GMimeObject *
 g_mime_parser_construct_part (GMimeParser *parser)
@@ -1602,13 +1619,13 @@ parser_construct_message (GMimeParser *parser)
 	
 	/* scan the from-line if we are parsing an mbox */
 	while (priv->state != GMIME_PARSER_STATE_HEADERS) {
-		if (parser_step (parser) == -1)
+		if (parser_step (parser) == GMIME_PARSER_STATE_ERROR)
 			return NULL;
 	}
 	
 	/* parse the headers */
 	while (priv->state < GMIME_PARSER_STATE_HEADERS_END) {
-		if (parser_step (parser) == -1)
+		if (parser_step (parser) == GMIME_PARSER_STATE_ERROR)
 			return NULL;
 	}
 	
@@ -1621,7 +1638,7 @@ parser_construct_message (GMimeParser *parser)
 				content_length = ULONG_MAX;
 		}
 		
-		g_mime_object_add_header ((GMimeObject *) message, header->name, header->value);
+		g_mime_object_append_header ((GMimeObject *) message, header->name, header->value);
 		header = header->next;
 	}
 	
@@ -1656,7 +1673,7 @@ parser_construct_message (GMimeParser *parser)
  *
  * Constructs a MIME message from @parser.
  *
- * Returns a MIME message or %NULL on fail.
+ * Returns: a MIME message or %NULL on fail.
  **/
 GMimeMessage *
 g_mime_parser_construct_message (GMimeParser *parser)
@@ -1674,7 +1691,7 @@ g_mime_parser_construct_message (GMimeParser *parser)
  * Gets the mbox-style From-line of the most recently parsed message
  * (gotten from g_mime_parser_construct_message()).
  *
- * Returns the mbox-style From-line of the most recently parsed
+ * Returns: the mbox-style From-line of the most recently parsed
  * message or %NULL on error.
  **/
 char *
@@ -1702,10 +1719,10 @@ g_mime_parser_get_from (GMimeParser *parser)
  * Gets the offset of the most recently parsed mbox-style From-line
  * (gotten from g_mime_parser_construct_message()).
  *
- * Returns the offset of the most recently parsed mbox-style From-line
+ * Returns: the offset of the most recently parsed mbox-style From-line
  * or -1 on error.
  **/
-off_t
+gint64
 g_mime_parser_get_from_offset (GMimeParser *parser)
 {
 	struct _GMimeParserPrivate *priv;
